# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-05-11 19:19
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djasana', '0001_squashed_0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attachment',
            name='download_url',
            field=models.URLField(max_length=1024),
        ),
        migrations.AlterField(
            model_name='attachment',
            name='name',
            field=models.CharField(max_length=1024, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='attachment',
            name='permanent_url',
            field=models.URLField(max_length=1024),
        ),
        migrations.AlterField(
            model_name='attachment',
            name='view_url',
            field=models.URLField(max_length=1024),
        ),
        migrations.AlterField(
            model_name='project',
            name='name',
            field=models.CharField(max_length=1024, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='story',
            name='name',
            field=models.CharField(max_length=1024, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='story',
            name='num_hearts',
            field=models.SmallIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='story',
            name='text',
            field=models.CharField(max_length=1024),
        ),
        migrations.AlterField(
            model_name='tag',
            name='name',
            field=models.CharField(max_length=1024, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='task',
            name='assignee_status',
            field=models.CharField(choices=[('inbox', 'inbox'), ('upcoming', 'upcoming'), ('later', 'later')], max_length=16),
        ),
        migrations.AlterField(
            model_name='task',
            name='name',
            field=models.CharField(max_length=1024, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='task',
            name='num_hearts',
            field=models.SmallIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='team',
            name='name',
            field=models.CharField(max_length=1024, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='user',
            name='name',
            field=models.CharField(max_length=1024, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='workspace',
            name='name',
            field=models.CharField(max_length=1024, verbose_name='name'),
        ),
    ]
